"""REST infrastructure to simplify communicating with REST based iLO interfaces

   Helper module to work with REST based APIs of BMCs.
"""

from proliantutils.rest.v1 import RestConnectorBase  # noqa
