# Copyright 2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

from sushy.resources.chassis import chassis
from sushy import utils as sushy_utils

from proliantutils.redfish.resources.chassis import devices
from proliantutils.redfish import utils


class HPEChassis(chassis.Chassis):
    """Class that extends the functionality of Chassis resource class

    This class extends the functionality of Chassis resource class
    from sushy
    """

    @property
    @sushy_utils.cache_it
    def devices(self):
        return devices.DevicesCollection(
            self._conn, utils.get_subresource_path_by(
                self, ['Oem', 'Hpe', 'Links', 'Devices']),
            redfish_version=self.redfish_version)
